/*
Copyright © 2020-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage
 Licensees holding valid commercial IoT.bzh licenses may use this file in
 accordance with the commercial license agreement provided with the
 Software or, alternatively, in accordance with the terms contained in
 a written agreement between you and The IoT.bzh Company. For licensing terms
 and conditions see https://www.iot.bzh/terms-conditions. For further
 information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage
 Alternatively, this file may be used under the terms of the GNU General
 Public license version 3. This license is as published by the Free Software
 Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
 of this file. Please review the following information to ensure the GNU
 General Public License requirements will be met
 https://www.gnu.org/licenses/gpl-3.0.html.
$RP_END_LICENSE$
*/

package cmd

const (
	localAutogenSubDirectory = ".redpesk"
	localIotpkgSubDirectory  = ".iotpkg"
	localMD5SUMFile          = "md5sum.json"

	localV2LocalBldLogsTag1 = "\nLOCAL-BUILDER LOGS >>>"
	localV2LocalBldLogsTag2 = "<<< LOCAL-BUILDER LOGS\n"

	// --------------------------------------------------------------
	// Subcommands and their subcommands ----------------------------
	// --------------------------------------------------------------

	// Main command
	localV2Command      = "local-v2"
	localV2CmdShortDesc = "Local commands group v2"
	// -------------

	// Package init sub-command
	localV2PkgInitBranchFlag      = "branch"
	localV2PkgInitBranchFlagAlias = "b"
	localV2PkgInitBranchFlagUsage = "[Optional] Specify the package branch to build locally"

	localV2PkgInitForceFlag      = "force"
	localV2PkgInitForceFlagAlias = "f"
	localV2PkgInitForceFlagUsage = "[Optional] Overwrite any already existing gitpkgs/gitsources repository for this application, no check will be done"

	localV2PkgInitAnsLine0 = "Check 'gitsources' repository status...\t"
	localV2PkgInitAnsLine1 = "Remove old 'gitsources' repository...\t"
	// -------------

	// Fetch command
	localV2FetchCmd          = "fetch <app-ID>"
	localV2FetchCmdShortDesc = "Fetch content from the redpesk factory, create or update a local working directory"
	localV2FetchCmdLongDesc  = `Fetch content from the redpesk factory, create or update a local working directory.
The content fetch is the application 'gitpkg' repository and the local-builder configuration, mandatory to build locally.
Only one type of content can be downloaded by specifying one optional flag.`

	localV2FetchCmdGitpkgFlag      = "gitpkg"
	localV2FetchCmdGitpkgFlagAlias = "g"
	localV2FetchCmdGitpkgFlagUsage = "[Optional] Download only the 'gitpkg' content"

	localV2FetchCmdMockCfgFlag      = "local-builder-config"
	localV2FetchCmdMockCfgFlagAlias = "c"
	localV2FetchCmdMockCfgFlagUsage = "[Optional] Download only the local builder configuration"

	localV2FetchCmdForceFlag      = "force"
	localV2FetchCmdForceFlagAlias = "f"
	localV2FetchCmdForceFlagUsage = "[Optional] Overwrite any already existing gitpkg repository for this application, no check will be done"

	localV2FetchCmdAnsLine0 = "\n-- Content fetching requested by user --\n"
	localV2FetchCmdAnsLine1 = "Retrieve application details...\t"
	localV2FetchCmdAnsLine2 = "Check local-builder configuration...\t"
	localV2FetchCmdAnsLine3 = "Create directories if needed...\t"
	localV2FetchCmdAnsLine4 = "Check 'gitpkg' repository status...\t"
	localV2FetchCmdAnsLine5 = "Fetch the 'gitpkg' repository from branch '%s'...\t"
	localV2FetchCmdAnsLine6 = "Fetch local-builder configuration...\t"
	// -------------

	// Status command
	localV2StatusCmd          = "status <app-ID>"
	localV2StatusCmdShortDesc = "Show the status of files in a local working directory"
	localV2StatusCmdLongDesc  = `Show the status of files in a local working directory.
This command indicates wether files have been modified locally, deleted or added.`

	localV2StatusCmdAnsLine0 = "\n-- Content status requested by user --\n"
	localV2StatusCmdAnsLine1 = "Status:"
	localV2StatusCmdAnsLine2 = "On branch %s\n"

	localV2StatusAddedFileLine    = "    added:    %s\n"
	localV2StatusDelFileLine      = "    deleted:  %s\n"
	localV2StatusModifiedFileLine = "    modified: %s\n"
	localV2StatusNoChanges        = "    local working directory clean, no changes detected"
	// -------------

	// Commit command
	localV2CommitCmd          = "commit <app-ID> [files...]"
	localV2CommitCmdShortDesc = "Upload files that have been modified/added/deleted in your working directory, to the remote gitpkg in the redpesk factory"
	localV2CommitCmdLongDesc  = `Upload files that have been modified/added/deleted in your working directory, to the remote gitpkg in the redpesk factory.
If no files are specified, rp-cli upload any modified/added/deleted from the working directory.`

	localV2CommitCmdAnsLine0 = "\n-- Commit requested by user --\n"

	localV2CommitNothingToCommit = "Nothing to commit"
	localV2CommitUpload          = "Upload of '%s'...\n"
	localV2CommitRemoval         = "Removal of '%s'...\n"
	// -------------

	// Service command
	localV2ServiceCmd          = "service <app-ID> <service-name>"
	localV2ServiceCmdShortDesc = "Set application service value"

	localV2ServiceCmdAnsLine0 = "\n-- Service change requested by user --\n"

	// Metadata command
	localV2MetadataCmd          = "metadata <app-ID>"
	localV2MetadataCmdShortDesc = "Set application metadata value"

	localV2MetadataCmdAnsLine0 = "\n-- Metadata change requested by user --\n"

	// Build command
	localV2BuildCmd           = "build <app-ID>"
	localV2BuildCmdShortUsage = "Build locally an application"
	localV2BuildCmdLongUsage  = "Build locally the application <app-ID>"

	localV2BuildProjectFlag      = "project"
	localV2BuildProjectFlagAlias = "p"
	localV2BuildProjectFlagUsage = "[Optional] Specify the project's slug where the application <app-ID> is or will be"

	localV2BuildArchFlag        = "arch"
	localV2BuildArchFlagAlias   = "a"
	localV2BuildArchFlagUsage   = "[Optional] Target architecture"
	localV2BuildArchFlagDefault = "aarch64"

	localV2BuildDistroFlag      = "distro"
	localV2BuildDistroFlagAlias = "d"
	localV2BuildDistroFlagUsage = "[Optional] Target distribution"

	localV2BuildRunCmdAnsLine0      = "\n-- Build requested by user --\n"
	localV2BuildRunCmdAnsLine1      = "Building the application..."
	localV2BuildRunCmdAnsResultGood = "Congratulations! Your build seems to be successful!"
	localV2BuildRunCmdAnsResultBad  = "Oops... A wild error appeared while building your application!"

	// Shell command
	localV2ShellCmd           = "shell <app-ID>"
	localV2ShellCmdShortUsage = "Allow to get a shell inside the mock used to build"
	localV2ShellCmdLongUsage  = "Allow to get a shell inside the mock used to build the app '<app-ID>'"

	localV2ShellCmdAnsLine0 = "\n-- Shell requested by user --\n"

	// Clean command
	localV2CleanNoMockFlag      = "no-mock"
	localV2CleanNoMockFlagAlias = "n"
	localV2CleanNoMockFlagUsage = "[Optional] If set, the mock directory is not deleted inside the local-builder"

	// Check-setup command
	localV2CheckSetupCmd           = "check-setup"
	localV2CheckSetupCmdShortUsage = "Check that the host is correctly set-up to use the local-builder"

	localV2CheckSetupCmdAnsLine0 = "\n-- Local check requested by user --\n"
	localV2CheckSetupCmdAnsLine1 = "Check factory connection...\t"
	localV2CheckSetupCmdAnsLine2 = "Check local-builder connection...\t"
	localV2CheckSetupCmdAnsLine3 = "Check local-builder mount-points...\t"
	localV2CheckSetupCmdAnsLine4 = "Check QEMU set-up...\t"

	// --------------------------------------------------------------
	// --------------------------------------------------------------

	// --------------------------------------------------------------
	// Messages -----------------------------------------------------
	// --------------------------------------------------------------

	// Information messages
	// -------------

	// Error messages
	md5sumFileDoesNotExist      = "no md5sum file found for this application"
	cannotFetchWithChanges      = "cannot fetch because you have local changes, please commit or use the --force flag"
	cannotCloneWithChanges      = "cannot clone because you have local changes, please commit or use the --force flag"
	needToSpecifyDistroErr      = "several distributions have been found for the project \"%s\", please specify one with the '-d' flag"
	needToChangeDistroErr       = "the distribution specified (\"%s\") does not exists for this project (\"%s\"), please modify the distribution wanted"
	needToChangeArchErr         = "the architecture specified (\"%s\") does not exists for this project (\"%s\"), please modify the architecture wanted"
	pbOnMountPoints             = "a problem has been seen on the mounted directories inside the local-builder, either on the directories themselves or on the UID/GID"
	varLibBinfmtsFileNotGood    = "the content of /var/lib/binfmts/qemu-aarch64 does not seems good"
	procSysFsBinfmtsFileNotGood = "the content of /proc/sys/fs/binfmt_misc/qemu-aarch64 does not seem good, please try to follow the steps available here: https://docs.redpesk.bzh/docs/en/master/redpesk-factory/local_builder/docs/4_advanced-installation.html#setup-your-host-for-local-crossbuild"
	// -------------

	// Warning messages

	// -------------

	// Debug messages

	// -------------

	// --------------------------------------------------------------
	// --------------------------------------------------------------

	// --------------------------------------------------------------
	// Files content for diagnosis ----------------------------------
	// --------------------------------------------------------------
	varLibBinfmtsQemuAarch64 = `qemu-user-static
magic
0
\x7f\x45\x4c\x46\x02\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\xb7\x00
\xff\xff\xff\xff\xff\xff\xff\x00\xff\xff\xff\xff\xff\xff\xff\xff\xfe\xff\xff\xff
/usr/bin/qemu-aarch64-static`

	procSysFsBinfmtMiscQemuAarch64V0 = `enabled
interpreter /usr/bin/qemu-aarch64-static
flags: F
offset 0
magic 7f454c460201010000000000000000000200b700
mask ffffffffffffff00fffffffffffffffffeffffff`

	procSysFsBinfmtMiscQemuAarch64V1 = `enabled
interpreter /usr/bin/qemu-aarch64-static
flags: OCF
offset 0
magic 7f454c460201010000000000000000000200b700
mask ffffffffffffff00fffffffffffffffffeffffff`

	fixVarLibBinfmtsQemuAarch64file = `sudo cat << EOF | sudo tee /var/lib/binfmts/qemu-aarch64
 qemu-user-static
 magic
 0
 \x7f\x45\x4c\x46\x02\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\xb7\x00
 \xff\xff\xff\xff\xff\xff\xff\x00\xff\xff\xff\xff\xff\xff\xff\xff\xfe\xff\xff\xff
 /usr/bin/qemu-aarch64-static



 yes
 EOF`

	// --------------------------------------------------------------
	// --------------------------------------------------------------
)

// Type used to store information about gitpkg files modifications
type RPLocalGitpkgsModification struct {
	Added    []string
	Modified []string
	Deleted  []string
}

// Structure used to contain all the data necessary to
// set the local builder for build
type RPLocalBuildConfigV2 struct {
	SourcesDir  string `json:"sources_dir"`
	PackagesDir string `json:"packages_dir"`
	Branch      string `json:"branch"`
}
