/*
Copyright © 2020-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage
 Licensees holding valid commercial IoT.bzh licenses may use this file in
 accordance with the commercial license agreement provided with the
 Software or, alternatively, in accordance with the terms contained in
 a written agreement between you and The IoT.bzh Company. For licensing terms
 and conditions see https://www.iot.bzh/terms-conditions. For further
 information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage
 Alternatively, this file may be used under the terms of the GNU General
 Public license version 3. This license is as published by the Free Software
 Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
 of this file. Please review the following information to ensure the GNU
 General Public License requirements will be met
 https://www.gnu.org/licenses/gpl-3.0.html.
$RP_END_LICENSE$
*/

package cmd

const (
	// Paths inside the local builder
	localBuilderHomeDir       = "/home/devel"
	localBuilderGitSourcesDir = localBuilderHomeDir + "/gitsources"
	localBuilderGitPkgsDir    = localBuilderHomeDir + "/gitpkgs"

	// Configuration directory on the host
	localCfgSubDirectory = ".rp-cli"

	localPrjNotSpecified = "project-not-specified"

	// --------------------------------------------------------------
	// Subcommands and their subcommands ----------------------------
	// --------------------------------------------------------------

	// Main command
	localCommand      = "local"
	localCmdShortDesc = "Local commands group"
	// -------------

	// Execute command
	localExecCmdName       = "exec -- <cmdLine>"
	localExecCmdShortUsage = "Execute a command line in the local-builder"
	localExecCmdLongUsage  = "Execute the command line <cmdLine> in the local-builder"

	localExecCmdWorkingDirFlag      = "working-directory"
	localExecCmdWorkingDirFlagAlias = "w"
	localExecCmdWorkingDirFlagUsage = "Set the working directory, in local-builder, to run the command"
	// -------------

	// Package sub-commands
	localPkgCmdName       = "package"
	localPkgCmdAlias      = "pkg"
	localPkgCmdShortUsage = "Packaging sub-command group"

	localPkgCmdSourceFlag      = "sourcesdir"
	localPkgCmdSourceFlagAlias = "S"
	localPkgCmdSourceFlagUsage = "[Optional] \"gitsources/\" directory path on the host"

	localPkgCmdPackageFlag      = "pkgsdir"
	localPkgCmdPackageFlagAlias = "P"
	localPkgCmdPackageFlagUsage = "[Optional] \"gitpkgs/\" directory path on the host"
	// -------------

	// Package init sub-command
	localPkgInitCmdName       = "init <app-ID>"
	localPkgInitCmdShortUsage = "Initialization needed by the local builder to package"
	localPkgInitCmdLongUsage  = "Initialization needed by the local builder to package the application named <app-ID>"

	localPkgInitProjectFlag      = "project"
	localPkgInitProjectFlagUsage = "[Optional] Specify the project's slug where the application <app-ID> is"

	localPkgInitSpecPathFlag      = "spec-path"
	localPkgInitSpecPathFlagUsage = "[Optional] URL or relative path (from the root of the sources) to the spec file (if app not created yet in Redpesk)"

	localPkgInitSpecLocalFlag      = "spec-local"
	localPkgInitSpecLocalFlagUsage = "[Optional] Path to the specfile locally. If set, this value overrides \"spec-path\" value"

	localPkgInitSrcURLFlag      = "source-url"
	localPkgInitSrcURLFlagUsage = "[Optional] GIT repository's URL of the package sources (if app not created yet in Redpesk)"

	localPkgInitSrcRevFlag      = "repo-rev"
	localPkgInitSrcRevFlagUsage = "[Optional] Revision of the GIT repository containing the sources or the archive + specfile"

	localPkgInitPkgURLFlag      = "package-url"
	localPkgInitPkgURLFlagUsage = `[Optional] GIT repository's URL of the package sources (if app not created yet in Redpesk).
This can be used when the user does not have access to the application sources`

	localPkgInitOfflineFlag      = "offline"
	localPkgInitOfflineFlagUsage = "[Optional] Specify that the packaging needs to be run offline afterwards"

	localPkgInitCmdAnsLine0    = "\n-- Package initialization requested by user --\n"
	localPkgInitCmdAnsLine1    = "Checking application details...\t"
	localPkgInitCmdAnsLine2    = "Checking packaging directories...\t"
	localPkgInitCmdAnsLine3    = "\"gitsources/\" directory used: \t%s\n"
	localPkgInitCmdAnsLine4    = "\"gitpkgs/\" directory used: \t%s\n"
	localPkgInitCmdAnsLine5    = "Retrieving package configuration for \"%s\"\t"
	localPkgInitCmdAnsLine6    = "Setting up the local-builder..."
	localPkgInitCmdAnsLine7    = "Cloning the package sources..."
	localPkgInitCmdAnsLine8    = "Working out the spec file...\t"
	localPkgInitCmdAnsLine9    = "The spec file '%s' has been copied to '%s'\n"
	localPkgInitCmdAnsLine10   = "Please modify '%s' while working on packaging, only this specfile is used when running 'rp-cli local package build'\n"
	localPkgInitCmdAnsLine11   = "Creation of the sources archives inside local builder..."
	localPkgInitCmdAnsLine12   = "Please modify your specfile located in '%s' while working on packaging, it will be used by 'rp-cli local package build'\n"
	localPkgInitCmdAnsLine13   = "Retrieving build mock configurations from redpesk factory...\t"
	localPkgInitCmdAnsLine14   = "Retrieving target tag(s) from redpesk factory...\t"
	localPkgInitCmdAnsClone1   = "\nCLONE LOGS >>>"
	localPkgInitCmdAnsClone2   = "<<< CLONE LOGS\n"
	localPkgInitCmdAnsArchive1 = "\nARCHIVE LOGS >>>"
	localPkgInitCmdAnsArchive2 = "<<< ARCHIVE LOGS\n"
	// -------------

	// Package run sub-command
	localPkgBuildCmdName     = "build <app-ID>"
	localPkgRunCmdName       = "run"
	localPkgRunCmdShortUsage = "Run the packaging of an application"
	localPkgRunCmdLongUsage  = "Run the packaging of the application <app-ID>"

	localPkgRunCmdRegenArchFlag      = "regen-archive"
	localPkgRunCmdRegenArchFlagUsage = "[Optional] Force rp-cli to recreate the sources archive before packaging"

	localPkgRunProjectFlag      = "project"
	localPkgRunProjectFlagUsage = "[Optional] Specify the project's slug where the application <app-ID> is or will be"

	localPkgRunForceReleaseFlag      = "force-release"
	localPkgRunForceReleaseFlagUsage = "[Hidden Flag] Force the released named to be passed to iotpkg"

	localPkgRunTargetFlag      = "target"
	localPkgRunTargetFlagUsage = "[Optional] Target for packaging (x86_64 or aarch64)"

	localPkgRunCmdNoDelFlag      = "no-delete"
	localPkgRunCmdNoDelFlagAlias = "N"
	localPkgRunCmdNoDelFlagUsage = "[Optional] Package without deleting the mock (before and after)"

	localPkgRunCmdNoCleanAfterFlag      = "no-clean-after"
	localPkgRunCmdNoCleanAfterFlagUsage = "[Optional] Package without deleting the mock (after)"

	localPkgRunCmdInstallFlag      = "install"
	localPkgRunCmdInstallFlagUsage = "[Optional] Install manually a package inside the mock"

	localPkgRunCmdShellFlag      = "shell"
	localPkgRunCmdShellFlagUsage = "[Optional] Allow to get a shell inside the mock used to package (the mock needs to not have been deleted)"

	localPkgRunCmdDistroFlag      = "distribution"
	localPkgRunCmdDistroFlagAlias = "d"
	localPkgRunCmdDistroFlagUsage = "[Optional] Specify the distribution for which the packaging needs to be done"

	localPkgRunCmdAnsLine0 = "\n-- Packaging requested by user --\n"
	localPkgRunCmdAnsLine1 = "Re-generating archive..."
	localPkgRunCmdAnsLine2 = "Packaging the application..."
	localPkgRunCmdAnsLine3 = "Retrieving packaging parameters for %s...\t"
	localPkgRunCmdAnsLine4 = "Results and/or logs in:"
	localPkgRunCmdAnsLine5 = "Congratulations! Your packaging seems to be successful!"
	localPkgRunCmdAnsLine6 = "Oups... An error appeared while packaging your application"
	localPkgRunCmdAnsLine7 = "You can add easily your application in Redpesk factory using the following command:"
	localPkgRunCmdAnsLine8 = "[Warning] No archive regeneration possible in \"package only\" mode"
	localPkgRunCmdAnsLine9 = "[Warning] %d specfile(s) has been found in %s therefore rp-cli cannot display some useful information"
	localPkgRunCmdAnsPkg1  = "\nPACKAGING LOGS >>>"
	localPkgRunCmdAnsPkg2  = "<<< PACKAGING LOGS\n"

	localPkgRunAddAppTemplate = "rp-cli applications add -n %s --pkg-name %s --source-url %s --source-rev %s -p %s --specfile-type %s --specfile-location %s"
	// -------------

	// Package clean sub-command
	localPkgCleanCmdName       = "clean <app-ID>"
	localPkgCleanCmdShortUsage = "Delete all the directories created for local building"
	localPkgCleanCmdLongUsage  = "Delete all the directories created for local building of the application <app-ID>"

	localPkgCleanInitFlag      = "init"
	localPkgCleanInitFlagAlias = "i"
	localPkgCleanInitFlagUsage = "[Optional] If set, rp-cli initializes a new clean directory with configurations found in the previous one"

	localPkgCleanProjectFlag      = "project"
	localPkgCleanProjectFlagUsage = "[Optional] Specify the application <app-ID> parent project slug"

	localPkgCleanForceFlag      = "force"
	localPkgCleanForceFlagAlias = "f"
	localPkgCleanForceFlagUsage = "[Optional] If set, rp-cli clean directories without checking for sources modification"

	localPkgCleanCmdAnsLine0 = "-- Package cleaning requested by user --"
	localPkgCleanCmdAnsLine1 = "There are some modifications inside your gitsources. Are you sure you want to delete? (Y/n)"
	localPkgCleanCmdAnsLine2 = "CLEANED"
	// -------------

	// --------------------------------------------------------------
	// --------------------------------------------------------------

	// --------------------------------------------------------------
	// Events used --------------------------------------------------
	// --------------------------------------------------------------
	localStdOutEvt  = "stdout"
	localStdErrEvt  = "stderr"
	localStdInEvt   = "stdin"
	localProcEndEvt = "process_ended"
	// --------------------------------------------------------------
	// --------------------------------------------------------------

	// --------------------------------------------------------------
	// Messages -----------------------------------------------------
	// --------------------------------------------------------------

	// Information messages
	// -------------

	// Error messages
	cmdToShortError              = "no command found to run. Format: \"rp-cli local exec -- <cmdline>\""
	stdinErrorOccured            = "a problem occured while reading from stdin"
	stdShouldBeTerminal          = "stdin/stdout should be terminal"
	execErrorMsg                 = "the following error happened while running exec command: %v"
	getTermSizeError             = "an error occured while getting the terminal size: %v"
	webSocketDisconnectionErr    = "the websocket between rp-cli and the local-builder has been disconnected"
	noSourcesURLError            = "no sources or package URL specified... Please provide one"
	duplicateSrcURL              = "URL is specified for sources and package, please provide only one "
	noSpecPathError              = "no URL or path specified for the spec file... Please provide one"
	noLocalSpecFileError         = "the local spec file specified does not exists"
	dirExistsError               = "the directory \"%s\" already exists, please delete it and replay your command"
	specScenarioNotManagedErr    = "this scenario of spec file is not managed"
	noReleaseGotErr              = "no release value has been retrieved from project"
	noDistroInProjectErr         = "the distribution \"%s\" has not be found in project \"%s\""
	targetNotSupportedErr        = "this target is not supported, please use another one"
	tooManyBuildConfFileFoundErr = "too many build configuration files have been found for this build"
	NoConfFileFoundErr           = "no \"build\" configuration file have been found for the distro \"%s\" and the arch \"%s\""
	gitPkgNotInstalled           = "please install 'git' package to be able to run the 'rp-cli local package init command"
	gitLFSPkgNotInstalled        = "please install 'git-lfs' package to be able to run the 'rp-cli local package init command"
	packageCleanDirNotFound      = "no directory to clean, path used: '%s'"
	// -------------

	// Warning messages
	localCleanNoSourcesDirWarn = "No directory found for sources of app '%s'"
	noGitSourceFoundWarn       = "The source directory may not be a git repository, error: '%v'"
	// -------------

	// Debug messages
	symlinkReqToLad     = "Asking redpesklad to create a symlink -- target: %s - symlink: %s"
	endOfStdinListening = "Stop listening on stdin"
	endOfProc           = "End of process %s with status %d"
	endOfProcStrStatus  = "End of process %s with status %s"
	// -------------

	// --------------------------------------------------------------
	// --------------------------------------------------------------
)

// Type used for local builder events
type RPLocalBuilderEvent struct {
	ID     string `json:"id"`
	Line   string `json:"std_line"`
	Status int    `json:"status"`
	Type   string `json:"type"`
}

// Type used to send stdin event to local builder
type RPLocalBStdinEvent struct {
	ID    string `json:"id"`
	Chars string `json:"stdin_chars"`
}

// Type used to notify the end of interactive command
type RPLocalExitResult struct {
	Error error
	Code  int
}

// Structure used to contain all the data necessary to
// set the local builder for build
type RPLocalBuildConfig struct {
	SourcesDir   string            `json:"sources_dir"`
	PackagesDir  string            `json:"packages_dir"`
	SpecFilePath string            `json:"spec_file_path"`
	IsSpecLocal  bool              `json:"is_spec_local"`
	IsSpecExt    bool              `json:"is_spec_ext"`
	SourcesURL   string            `json:"sources_url"`
	PackageURL   string            `json:"package_url"`
	SourcesRev   string            `json:"sources_revision"`
	OfflineMode  bool              `json:"offline_mode"`
	ReleasedTags map[string]string `json:"released_tags"`
	Branch       string            `json:"branch"`
}
